/*
   Class Name:

      CGLFont.

   Created by:

      Allen Sherrod (Programming Ace of www.UltimateGameProgramming.com).

   Description:

      This class will allow us to print text to the screen.  This class is used
      only with OpenGL for output purposes.
*/


#include<windows.h>              // The windows header file.
#include<gl/gl.h>                // Standard opengl include.
#include"CGLFont.h"              // Our font class header.


CGLFont::CGLFont()
{
   Width = Height = 400;
}


CGLFont::~CGLFont()
{
   ClearGLFont();                // Clear the font that we created in the constructor.
}


// Initialize font.
bool CGLFont::Initialize(HDC hDC, int Size, char *fontName)
{
   // Create the font list.
   listBase = InitFont(fontName, Size, hDC);

   if(listBase == 0)
      return false;

   return true;
}


unsigned int CGLFont::InitFont(char *fontName, int Size, HDC hDC)
{
   // Windows font
   HFONT hFont;
   unsigned int base;

   // Create space for 96 characters.
   base = glGenLists(96);

   if(stricmp(fontName, "symbol")==0)
      {
         hFont = CreateFont(Size, 0, 0, 0, FW_BOLD, FALSE, FALSE, FALSE,
                            SYMBOL_CHARSET, OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS,
                            ANTIALIASED_QUALITY, FF_DONTCARE | DEFAULT_PITCH, fontName);
      }
   else
      {
         hFont = CreateFont(Size, 0, 0, 0, FW_BOLD, FALSE, FALSE, FALSE,
                            ANSI_CHARSET, OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS,
                            ANTIALIASED_QUALITY, FF_DONTCARE | DEFAULT_PITCH, fontName);
      }

   // Error checking.
   if(!hFont)
      return 0;

   // Set up font.
   SelectObject(hDC, hFont);
   wglUseFontBitmaps(hDC, 32, 96, base);

   return base;
}


// Print the text at the desired position.
bool CGLFont::PrintText(char *string, float x, float y)
{
   // Error checking...
   if((listBase == 0 || string == NULL))
      return false;

   glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	   glLoadIdentity();

	   glOrtho(0, 0, Width, Height, 0, 1);
   
	   glMatrixMode(GL_MODELVIEW);
	   glLoadIdentity();

      // And finally in order to get this to work correctly we must disable depth testing.
      // Since this is not 3D we can't do any depth testing.
      glDisable(GL_DEPTH_TEST);

      // More to screen position sent in to this function.
      glRasterPos2f(x, y);

      // Print the string to the screen.
      glPushAttrib(GL_LIST_BIT);
         glListBase(listBase - 32);
         glCallLists(strlen(string), GL_UNSIGNED_BYTE, string);
      glPopAttrib();

       // Set everything back to perpective below, don't forget to re-enable depth testing.
      glEnable(GL_DEPTH_TEST);

      glMatrixMode(GL_PROJECTION);
   glPopMatrix();											

	glMatrixMode(GL_MODELVIEW);

   return true;
}


void CGLFont::SetWidthHeight(int w, int h)
{
   Width = w;
   Height = h;
}


// Delete the font.
void CGLFont::ClearGLFont()
{
   // Delete font.
   if(listBase != 0)
      glDeleteLists(listBase, 96);
}


// Copyright September 2003
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com